/*
 * The MIT License
 *
 * Copyright 2012 Massive Dynamics.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package biz.massivedynamics.filoloci;

/**
 * An enhanced version of {@link ThreadLocal} that provides commonly needed features
 *
 * @param <T> The class type to be used
 * @author Cruz Julian Bishop
 * @since 0.9.0.0
 */
public class EnhancedThreadLocal<T> extends ThreadLocal<T> implements Cloneable {
    
    /**
     * Constructs a new {@link EnhancedThreadLocal} class
     * 
     * @since 0.9.0.0
     */
    public EnhancedThreadLocal() {
        super();
        
        //Set initialValue as null by default
        this.initialValue = null;
    }
    
    /**
     * Constructs a new {@link EnhancedThreadLocal} class with the specified initial value
     * 
     * @param initialValue The initial value to use
     * @since 0.9.0.0
     */
    public EnhancedThreadLocal(T initialValue) {
        super();
        
        //Implement the parameter in a smarter way!
        this.initialValue = initialValue;
    }
    
    /**
     * The initial value for this {@link EnhancedThreadLocal}
     * 
     * @since 0.9.0.0
     */
    private final T initialValue;
    
    /**
     * Gets the initial value for this {@link EnhancedThreadLocal}
     * 
     * @return The initial value
     * @since 0.9.0.0
     */
    @Override
    public final T initialValue() {
        return this.initialValue;
    }
    
    /**
     * Clones this {@link EnhancedThreadLocal} class
     * 
     * @since 0.9.0.0
     */
    @Override
    public EnhancedThreadLocal<T> clone() {
        //Due to the setup of the ThreadLocal class, we can't specify this in the constructor
        return new EnhancedThreadLocal<T>(this.get());
    }
    
    /**
     * Gets the hash code for this {@link EnhancedThreadLocal}
     * <p>
     * In this case, it is the hash code of the super class.
     * This is because data is not actually changed by this class
     * </p>
     * 
     * @return The hash code
     */
    @Override
    public int hashCode() {
        return super.hashCode();
    }
    
    /**
     * Checks to see if this {@link EnhancedThreadLocal} is equal to another {@link ThreadLocal} or T class
     * 
     * @param other The other {@link Object} to check against
     * @return True if equal, otherwise false
     * @since 0.9.0.0
     */
    @Override
    public boolean equals(Object other) {
        //See if it already returns true
        if (super.equals(other)) return true;
        
        //No? Okay.
        
        //First, see if both are null
        if (this.get() == null && other == null)
            return true;
        
        //Check to see if other is a ThreadLocal
        if (other instanceof ThreadLocal) {
            return (this.get().equals(((ThreadLocal) other).get()));
        }
        
        //Okay? Let's see if it's equal to the T type!
        try {
            //This probably won't work if other isn't T
            T otherT = (T) other;
            
            return this.get().equals(otherT);
            
        } catch (ClassCastException exception) {
            //Nothing to do here, whoop-de-doo
        }
        
        //Okay, can't figure out anything else. 
        return false;
        
    }
    
}
